USE WideWorldImporters;
GO

-- Sample Table
CREATE TABLE dbo.OrderChangeLog
(	OrderChangeLogID INT NOT NULL IDENTITY(1,1) CONSTRAINT PK_OrderChangeLog PRIMARY KEY CLUSTERED,
	OrderID INT NOT NULL,
	CustomerID INT NOT NULL,
	ModifiedDateTime DATETIME NOT NULL,
	ModifiedByPersonID INT NOT NULL,
	ChangeRequestAPIDetails VARCHAR(MAX) NOT NULL,
	BatchNumber INT NOT NULL,
	ColumnChanged VARCHAR(128) NOT NULL,
	OldValue VARCHAR(MAX) NULL,
	NewValue VARCHAR(MAX) NULL);
GO

-- Deletion of data older than 12 months:
DELETE TOP (1000) OrderChangeLog
FROM dbo.OrderChangeLog
WHERE OrderChangeLog.ModifiedDateTime <= DATEADD(MONTH, -12, GETUTCDATE());

WHILE @@ROWCOUNT > 0
BEGIN
	DELETE TOP (1000) OrderChangeLog
	FROM dbo.OrderChangeLog
	WHERE OrderChangeLog.ModifiedDateTime <= DATEADD(MONTH, -12, GETUTCDATE());
END

-- Archival of data older than 12 months to a table called dbo.OrderChangeLogArchive (created here):
CREATE TABLE dbo.OrderChangeLogArchive
(	OrderChangeLogID INT NOT NULL CONSTRAINT PK_OrderChangeLogArchive PRIMARY KEY CLUSTERED WITH (DATA_COMPRESSION = PAGE),
	OrderID INT NOT NULL,
	CustomerID INT NOT NULL,
	ModifiedDateTime DATETIME NOT NULL,
	ModifiedByPersonID INT NOT NULL,
	ChangeRequestAPIDetails VARCHAR(MAX) NOT NULL,
	BatchNumber INT NOT NULL,
	ColumnChanged VARCHAR(128) NOT NULL,
	OldValue VARCHAR(MAX) NULL,
	NewValue VARCHAR(MAX) NULL);
GO

-- Archival of data older than 12 months:
DELETE TOP (1000) OrderChangeLog
OUTPUT	DELETED.OrderChangeLogID,
		DELETED.OrderID,
		DELETED.CustomerID,
		DELETED.ModifiedDateTime,
		DELETED.ModifiedByPersonID,
		DELETED.ChangeRequestAPIDetails,
		DELETED.BatchNumber,
		DELETED.ColumnChanged,
		DELETED.OldValue,
		DELETED.NewValue
INTO dbo.OrderChangeLogArchive
WHERE OrderChangeLog.ModifiedDateTime <= DATEADD(MONTH, -12, GETUTCDATE());

WHILE @@ROWCOUNT > 0
BEGIN
	DELETE TOP (1000) OrderChangeLog
	OUTPUT	DELETED.OrderChangeLogID,
			DELETED.OrderID,
			DELETED.CustomerID,
			DELETED.ModifiedDateTime,
			DELETED.ModifiedByPersonID,
			DELETED.ChangeRequestAPIDetails,
			DELETED.BatchNumber,
			DELETED.ColumnChanged,
			DELETED.OldValue,
			DELETED.NewValue
	INTO dbo.OrderChangeLogArchive
	WHERE OrderChangeLog.ModifiedDateTime <= DATEADD(MONTH, -12, GETUTCDATE());
END

-- Create a partitioned version of the previously introduced table.

ALTER DATABASE WideWorldImporters ADD FILEGROUP WideWorldImporters_2022_fg;
ALTER DATABASE WideWorldImporters ADD FILEGROUP WideWorldImporters_2023_fg;
ALTER DATABASE WideWorldImporters ADD FILEGROUP WideWorldImporters_2024_fg;
ALTER DATABASE WideWorldImporters ADD FILEGROUP WideWorldImporters_2025_fg;
ALTER DATABASE WideWorldImporters ADD FILEGROUP WideWorldImporters_2026_fg;

ALTER DATABASE WideWorldImporters ADD FILE
	(NAME = WideWorldImporters_2022_data, 
        FILENAME = 'C:\SQLData\WideWorldImporters_2022_data.ndf',
	 SIZE = 100MB, MAXSIZE = UNLIMITED, FILEGROWTH = 500MB)
TO FILEGROUP WideWorldImporters_2022_fg;
ALTER DATABASE WideWorldImporters ADD FILE
	(NAME = WideWorldImporters_2023_data, 
        FILENAME = 'C:\SQLData\WideWorldImporters_2023_data.ndf',
	 SIZE = 100MB, MAXSIZE = UNLIMITED, FILEGROWTH = 500MB)
TO FILEGROUP WideWorldImporters_2023_fg;
ALTER DATABASE WideWorldImporters ADD FILE
	(NAME = WideWorldImporters_2024_data, 
        FILENAME = 'C:\SQLData\WideWorldImporters_2024_data.ndf',
	 SIZE = 100MB, MAXSIZE = UNLIMITED, FILEGROWTH = 500MB)
TO FILEGROUP WideWorldImporters_2024_fg;
ALTER DATABASE WideWorldImporters ADD FILE
	(NAME = WideWorldImporters_2025_data, 
        FILENAME = 'C:\SQLData\WideWorldImporters_2025_data.ndf',
	 SIZE = 100MB, MAXSIZE = UNLIMITED, FILEGROWTH = 500MB)
TO FILEGROUP WideWorldImporters_2022_fg;
ALTER DATABASE WideWorldImporters ADD FILE
	(NAME = WideWorldImporters_2026_data, 
        FILENAME = 'C:\SQLData\WideWorldImporters_2026_data.ndf',
	 SIZE = 100MB, MAXSIZE = UNLIMITED, FILEGROWTH = 500MB)
TO FILEGROUP WideWorldImporters_2026_fg;

CREATE PARTITION FUNCTION OrderChangeLogPartitioned_years_function (DATE)
AS RANGE RIGHT FOR VALUES
('2023-01-01', '2024-01-01', '2025-01-01', '2026-01-01');

CREATE PARTITION SCHEME fact_order_BIG_CCI_years_scheme
AS PARTITION fact_order_BIG_CCI_years_function
TO (WideWorldImporters_2022_fg, WideWorldImporters_2023_fg, 
    WideWorldImporters_2024_fg, WideWorldImporters_2025_fg, 
    WideWorldImporters_2026_fg);

CREATE TABLE dbo.OrderChangeLogPartitioned
(	OrderChangeLogID INT NOT NULL IDENTITY(1,1) CONSTRAINT PK_OrderChangeLogPartitioned PRIMARY KEY CLUSTERED,
	OrderID INT NOT NULL,
	CustomerID INT NOT NULL,
	ModifiedDateTime DATETIME NOT NULL,
	ModifiedByPersonID INT NOT NULL,
	ChangeRequestAPIDetails VARCHAR(MAX) NOT NULL,
	BatchNumber INT NOT NULL,
	ColumnChanged VARCHAR(128) NOT NULL,
	OldValue VARCHAR(MAX) NULL,
	NewValue VARCHAR(MAX) NULL)
ON fact_order_BIG_CCI_years_scheme (ModifiedDateTime);
GO

TRUNCATE TABLE dbo.OrderChangeLogPartitioned WITH (PARTITIONS(1));

ALTER INDEX PK_OrderChangeLogPartitioned ON dbo.OrderChangeLogPartitioned REBUILD PARTITION = 4 WITH (DATA_COMPRESSION = PAGE);

-- Example of retention by clearing out a large column that is no longer needed after a year:
UPDATE TOP (1000) OrderChangeLog
	SET ChangeRequestAPIDetails = NULL
FROM dbo.OrderChangeLog
WHERE OrderChangeLog.ModifiedDateTime <= DATEADD(MONTH, -12, GETUTCDATE())
AND OrderChangeLog.ChangeRequestAPIDetails IS NOT NULL;

WHILE @@ROWCOUNT > 0
BEGIN
	UPDATE TOP (1000) OrderChangeLog
		SET ChangeRequestAPIDetails = NULL
	FROM dbo.OrderChangeLog
	WHERE OrderChangeLog.ModifiedDateTime <= DATEADD(MONTH, -12, GETUTCDATE())
	AND OrderChangeLog.ChangeRequestAPIDetails IS NOT NULL;
END



-- Cleanup
DROP TABLE dbo.OrderChangeLog;
DROP TABLE dbo.OrderChangeLogArchive;
DROP TABLE dbo.OrderChangeLogPartitioned;